@@ -28,6 +28,7 @@
 #include "mailcheck.h"
 #include "clock.h"
 #include "pager.h"
+#include "showdesktop.h"
 #include "tasklist.h"
 
 static gboolean
@@ -49,6 +50,9 @@ genutil_factory (PanelApplet *applet,
 	if (!strcmp (iid, "OAFIID:GNOME_MailcheckApplet"))
 		retval = fill_mailcheck_applet(applet);
 
+	if (!strcmp (iid, "OAFIID:GNOME_ShowDesktopApplet"))
+		retval = fill_show_desktop_applet(applet);
+        
 	return retval;
 }
 
