@@ -1,3 +1,4 @@
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*- */
 /*
  * libwnck based pager applet.
  * (C) 2001 Alexander Larsson 
@@ -172,10 +173,11 @@ response_cb(GtkWidget * widget,int id, PagerData *pager)
 							     GNOME_PROGRAM_STANDARD_PROPERTIES, NULL);
 		}
 
-		egg_screen_help_display_desktop (
-			gtk_widget_get_screen (pager->applet),
+		egg_help_display_desktop_on_screen (
 			applet_program, "workspace-switcher",
-			"workspace-switcher", "workspacelist-prefs", &error);
+			"workspace-switcher", "workspacelist-prefs",
+			gtk_widget_get_screen (pager->applet),
+			&error);
 		if (error) {
 			GtkWidget *dialog;
 			dialog = gtk_message_dialog_new (GTK_WINDOW(widget),
@@ -371,7 +373,7 @@ fill_pager_applet(PanelApplet *applet)
 	error = NULL;
 	pager->n_rows = panel_applet_gconf_get_int (applet, "num_rows", &error);
 	if (error) {
-                g_printerr (_("Error loading num_rows value for workspace switcher applet: %s\n"),
+                g_printerr (_("Error loading num_rows value for Workspace Switcher: %s\n"),
                             error->message);
 		g_error_free (error);
                 /* leave current value */
@@ -382,7 +384,7 @@ fill_pager_applet(PanelApplet *applet)
 	error = NULL;
 	display_names = panel_applet_gconf_get_bool (applet, "display_workspace_names", &error);
 	if (error) {
-                g_printerr (_("Error loading display_workspace_names value for workspace switcher applet: %s\n"),
+                g_printerr (_("Error loading display_workspace_names value for Workspace Switcher: %s\n"),
                             error->message);
 		g_error_free (error);
                 /* leave current value */
@@ -397,7 +399,7 @@ fill_pager_applet(PanelApplet *applet)
 	error = NULL;
 	pager->display_all = panel_applet_gconf_get_bool (applet, "display_all_workspaces", &error);
 	if (error) {
-                g_printerr (_("Error loading display_all_workspaces value for workspace switcher applet: %s\n"),
+                g_printerr (_("Error loading display_all_workspaces value for Workspace Switcher: %s\n"),
                             error->message);
 		g_error_free (error);
                 /* leave current value */
@@ -483,10 +485,11 @@ display_help_dialog (BonoboUIComponent *uic,
 						     GNOME_PROGRAM_STANDARD_PROPERTIES, NULL);
 	}
 
-	egg_screen_help_display_desktop (
-		gtk_widget_get_screen (pager->applet),
+	egg_help_display_desktop_on_screen (
 		applet_program, "workspace-switcher",
-		"workspace-switcher", NULL, &error);
+		"workspace-switcher", NULL,
+		gtk_widget_get_screen (pager->applet),
+		&error);
 	if (error) {
 		GtkWidget *dialog;
 		dialog = gtk_message_dialog_new (NULL,
@@ -647,12 +650,8 @@ static void
 num_workspaces_value_changed (GtkSpinButton *button,
 			      PagerData       *pager)
 {
-	GConfClient* client;
-	
-	client = gconf_client_get_default ();
-
-	gconf_client_set_int (client, "/desktop/gnome/applications/window_manager/number_of_workspaces",
-			      gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (pager->num_workspaces_spin)), NULL);
+        wnck_screen_change_workspace_count (pager->screen,
+                                            gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (pager->num_workspaces_spin)));
 }
 
 static void 
@@ -661,40 +660,22 @@ workspace_name_edited (GtkCellRendererText *cell_renderer_text,
 		       const gchar         *new_text,
 		       PagerData           *pager)
 {
-	GConfClient* client;
-	GtkTreeIter iter;
-	char *str;
-	GSList *list;
+        const gint *indices;
+        WnckWorkspace *workspace;
+        GtkTreePath *p;
 
-	if (gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL (pager->workspaces_store),
-						 &iter, path)) {
-		gtk_list_store_set (pager->workspaces_store,
-				    &iter,
-				    0, new_text,
-				    -1);
-	}
+        p = gtk_tree_path_new_from_string (path);
+        indices = gtk_tree_path_get_indices (p);
+        workspace = wnck_screen_get_workspace (pager->screen,
+                                               indices[0]);
+        if (workspace != NULL)
+                wnck_workspace_change_name (workspace,
+                                            new_text);
+        else
+                g_warning ("Edited name of workspace %d which no longer exists",
+                           indices[0]);
 
-	
-	list = NULL;
-	gtk_tree_model_get_iter_root (GTK_TREE_MODEL (pager->workspaces_store), &iter);
-	do {
-		gtk_tree_model_get (GTK_TREE_MODEL (pager->workspaces_store),
-				    &iter,
-				    0, &str,
-				    -1);
-		list = g_slist_prepend (list, str);
-	} while (gtk_tree_model_iter_next (GTK_TREE_MODEL (pager->workspaces_store), &iter));
-	
-	list = g_slist_reverse (list);
-
-	client = gconf_client_get_default ();
-	gconf_client_set_list (client,  "/desktop/gnome/applications/window_manager/workspace_names",
-			       GCONF_VALUE_STRING,
-			       list,
-			       NULL);
-
-	g_slist_foreach (list, (GFunc)g_free, NULL);
-	g_slist_free (list);
+        gtk_tree_path_free (p);
 }
 
 static gboolean
