@@ -294,7 +294,7 @@ aligned_widget_change_params (AlignedWidget *aligned,
 			      BasePState state,
 			      gboolean hidebuttons_enabled,
 			      gboolean hidebutton_pixmaps_enabled,
-			      PanelBackType back_type,
+			      PanelBackgroundType back_type,
 			      char *pixmap_name,
 			      gboolean fit_pixmap_bg,
 			      gboolean stretch_pixmap_bg,
@@ -348,12 +348,12 @@ aligned_widget_change_align (AlignedWidget *aligned,
 				      basep->state,
 				      basep->hidebuttons_enabled,
 				      basep->hidebutton_pixmaps_enabled,
-				      panel->back_type,
-				      panel->back_pixmap,
-				      panel->fit_pixmap_bg,
-				      panel->stretch_pixmap_bg,
-				      panel->rotate_pixmap_bg,
-				      &panel->back_color);
+				      panel->background.type,
+				      panel->background.image,
+				      panel->background.fit_image,
+				      panel->background.stretch_image,
+				      panel->background.rotate_image,
+				      &panel->background.color);
 }
 
 void 
@@ -374,12 +374,12 @@ aligned_widget_change_align_edge (AlignedWidget *aligned,
 				      basep->state,
 				      basep->hidebuttons_enabled,
 				      basep->hidebutton_pixmaps_enabled,
-				      panel->back_type,
-				      panel->back_pixmap,
-				      panel->fit_pixmap_bg,
-				      panel->stretch_pixmap_bg,
-				      panel->rotate_pixmap_bg,
-				      &panel->back_color);
+				      panel->background.type,
+				      panel->background.image,
+				      panel->background.fit_image,
+				      panel->background.stretch_image,
+				      panel->background.rotate_image,
+				      &panel->background.color);
 }
 				 
 GtkWidget *
@@ -393,7 +393,7 @@ aligned_widget_new (gchar *panel_id,
 		    int sz,
 		    gboolean hidebuttons_enabled,
 		    gboolean hidebutton_pixmaps_enabled,
-		    PanelBackType back_type,
+		    PanelBackgroundType back_type,
 		    char *back_pixmap,
 		    gboolean fit_pixmap_bg,
 		    gboolean stretch_pixmap_bg,
