@@ -34,12 +34,16 @@
 #include "panel-applet-frame.h"
 #include "panel-gconf.h"
 #include "panel-util.h"
+#include "panel.h"
 #include "session.h"
 #include "applet.h"
 #include "panel-marshal.h"
+#include "panel-background.h"
 
 #define HANDLE_SIZE 10
 
+#define PANEL_STOCK_DONT_DELETE "panel-dont-delete"
+
 #undef PANEL_APPLET_FRAME_DEBUG
 
 struct _PanelAppletFramePrivate {
@@ -52,7 +56,6 @@ struct _PanelAppletFramePrivate {
 
 	gchar                          *iid;
 	gboolean			moving_focus_out;
-	gboolean			clean_remove;
 
 	GtkAllocation                   child_allocation;
 	GdkRectangle                    handle_rect;
@@ -113,7 +116,9 @@ popup_handle_remove (BonoboUIComponent *uic,
 		     PanelAppletFrame  *frame,
 		     const gchar       *verbname)
 {
-	AppletInfo *info = frame->priv->applet_info;
+	AppletInfo *info;
+
+	info = frame->priv->applet_info;
 	frame->priv->applet_info = NULL;
 
 	panel_applet_clean (info, TRUE);
@@ -166,13 +171,13 @@ panel_applet_frame_load (const gchar *iid,
 
 	frame = panel_applet_frame_new (panel, iid, real_key);
 
-	if (!frame)
+	if (!frame) {
+		g_free (real_key);
 		return;
+	}
 	
 	gtk_widget_show_all (frame);
 
-	/* Pass frame as 2nd argument, since it is used in
-	 * panel_remove_applets()  */
 	info = panel_applet_register (frame, frame, NULL, panel, position,
 				      exactpos, APPLET_BONOBO, real_key);
 
@@ -204,41 +209,45 @@ panel_applet_frame_get_expand_flags (PanelAppletFrame *frame,
 	*expand_minor = flags & APPLET_EXPAND_MINOR;
 }
 
-int *
-panel_applet_frame_get_size_hints (PanelAppletFrame *frame,
-				   int              *n_elements)
+int 
+panel_applet_frame_get_size_hints (PanelAppletFrame  *frame,
+				   int              **size_hints)
 {
-	CORBA_any *value;
 	CORBA_sequence_CORBA_long *seq;
-	int *size_hints;
-	int extra_size;
-	int i;
+	CORBA_any                 *value;
+	int                        retval = 0;
+	int                        extra_size = 0;
+	int                        i;
+
+
+	g_return_val_if_fail (PANEL_IS_APPLET_FRAME (frame), 0);
+	g_return_val_if_fail (size_hints != NULL, 0);
+
+	*size_hints = NULL;
 
-	*n_elements = 0;
 	value = bonobo_pbclient_get_value (frame->priv->property_bag,
 					   "panel-applet-size-hints",
 					   TC_CORBA_sequence_CORBA_long,
 					   NULL);
 	
 	if (value == NULL)
-		return NULL;
+		return retval;
 
 	seq = value->_value;
 
-	*n_elements = seq->_length;
-	size_hints = g_new (int, seq->_length);
+	retval = seq->_length;
+	*size_hints = g_new (int, seq->_length);
 
 	extra_size = 0;
 	if (panel_applet_frame_get_flags (frame) & APPLET_HAS_HANDLE)
 		extra_size = HANDLE_SIZE + 1;
 	
-	for (i = 0; i < seq->_length; i++) {
-		size_hints[i] = seq->_buffer[i] + extra_size;
-	}
+	for (i = 0; i < seq->_length; i++)
+		(*size_hints) [i] = seq->_buffer [i] + extra_size;
 	
 	CORBA_free (value);
 	
-	return size_hints;
+	return retval;
 }
 
 void
@@ -268,46 +277,19 @@ panel_applet_frame_change_size (PanelAppletFrame *frame,
 }
 
 static char *
-panel_applet_frame_get_background_string (PanelAppletFrame *frame,
-					  PanelWidget      *panel,
-					  PanelBackType     type)
+panel_applet_frame_get_background_string (PanelAppletFrame    *frame,
+					  PanelWidget         *panel,
+					  PanelBackgroundType  type)
 {
-	char *retval = NULL;
-
-	switch (type) {
-	case PANEL_BACK_PIXMAP: {
-		GdkNativeWindow pixmap_xid;
-
-		pixmap_xid = gdk_x11_drawable_get_xid (
-				GDK_DRAWABLE (GTK_WIDGET (panel)->window));
-
-		retval = g_strdup_printf (
-				"pixmap:%d,%d,%d", pixmap_xid,
-				GTK_WIDGET (frame)->allocation.x,
-				GTK_WIDGET (frame)->allocation.y);
-		}
-		break;
-	case PANEL_BACK_COLOR:
-		retval = g_strdup_printf (
-				"color:%.4x%.4x%.4x", 
-				panel->back_color.red, 
-				panel->back_color.green, 
-				panel->back_color.blue);
-		break;
-	case PANEL_BACK_NONE:
-		retval = g_strdup ("none:");
-		break;
-	default:
-		g_assert_not_reached ();
-		break;
-	}
-
-	return retval;
+	return panel_background_make_string (
+			&panel->background,
+			GTK_WIDGET (frame)->allocation.x,
+			GTK_WIDGET (frame)->allocation.y);
 }
 
 void
-panel_applet_frame_change_background (PanelAppletFrame *frame,
-				      PanelBackType     type)
+panel_applet_frame_change_background (PanelAppletFrame    *frame,
+				      PanelBackgroundType  type)
 {
 	char *bg_str;
 
@@ -331,19 +313,6 @@ panel_applet_frame_set_info (PanelAppletFrame *frame,
 	frame->priv->applet_info = info;
 }
 
-void
-panel_applet_frame_set_clean_remove (PanelAppletFrame *frame,
-				     gboolean          clean_remove)
-{
-	frame->priv->clean_remove = clean_remove;
-}
-
-gboolean
-panel_applet_frame_get_clean_remove (PanelAppletFrame *frame)
-{
-	return frame->priv->clean_remove;
-}
-
 static void
 panel_applet_frame_finalize (GObject *object)
 {
@@ -425,6 +394,33 @@ panel_applet_frame_expose (GtkWidget      *widget,
 	return FALSE;
 }
 
+static void
+panel_applet_frame_constrain_size (PanelAppletFrame *frame,
+				   GtkRequisition   *requisition)
+{
+	PanelWidget *panel;
+
+	g_return_if_fail (PANEL_IS_WIDGET (GTK_WIDGET (frame)->parent));
+
+	panel = PANEL_WIDGET (GTK_WIDGET (frame)->parent);
+
+	switch (frame->priv->orient) {
+	case PANEL_ORIENT_UP:
+	case PANEL_ORIENT_DOWN:
+		if (requisition->height > panel->sz)
+			requisition->height = panel->sz;
+		break;
+	case PANEL_ORIENT_LEFT:
+	case PANEL_ORIENT_RIGHT:
+		if (requisition->width > panel->sz)
+			requisition->width = panel->sz;
+		break;
+	default:
+		g_assert_not_reached ();
+		break;
+	}
+}
+
 static void
 panel_applet_frame_size_request (GtkWidget      *widget,
 				 GtkRequisition *requisition)
@@ -438,6 +434,7 @@ panel_applet_frame_size_request (GtkWidget      *widget,
 
 	if (!(panel_applet_frame_get_flags (frame) & APPLET_HAS_HANDLE)) {
 		GTK_WIDGET_CLASS (parent_class)->size_request (widget, requisition);
+		panel_applet_frame_constrain_size (frame, requisition);
 		return;
 	}
   
@@ -467,6 +464,8 @@ panel_applet_frame_size_request (GtkWidget      *widget,
 		g_assert_not_reached ();
 		break;
 	}
+
+	panel_applet_frame_constrain_size (frame, requisition);
 }
 
 static void
@@ -625,11 +624,19 @@ panel_applet_frame_reload_response (GtkWidget        *dialog,
 				    int               response,
 				    PanelAppletFrame *frame)
 {
-	AppletInfo *info = frame->priv->applet_info;
+	AppletInfo *info;
 
-	if (response == GTK_RESPONSE_NO)
-		panel_applet_clean (info, TRUE);
-	else {
+	g_return_if_fail (PANEL_IS_APPLET_FRAME (frame));
+
+	if (!GTK_WIDGET (frame)->parent) {
+		g_object_unref (frame);
+		gtk_widget_destroy (dialog);
+		return;
+	}
+
+	info = frame->priv->applet_info;
+
+	if (response == GTK_RESPONSE_YES) {
 		PanelWidget *panel;
 		char        *iid;
 		char        *gconf_key;
@@ -646,8 +653,10 @@ panel_applet_frame_reload_response (GtkWidget        *dialog,
 
 		g_free (iid);
 		g_free (gconf_key);
-	}
+	} else
+		panel_applet_clean (info, TRUE);
 
+	g_object_unref (frame);
 	gtk_widget_destroy (dialog);
 }
 
@@ -691,9 +700,16 @@ static void
 panel_applet_frame_cnx_broken (PanelAppletFrame *frame)
 {
 	GtkWidget *dialog;
+	GdkScreen *screen;
 	char      *applet_name;
 	char      *txt;
 
+	g_return_if_fail (frame->priv->applet_info != NULL);
+
+	screen = gtk_window_get_screen (
+			GTK_WINDOW (get_panel_parent (
+					frame->priv->applet_info->widget)));
+
 	applet_name = panel_applet_frame_get_name (frame->priv->iid);
 
 	txt = g_strdup_printf (
@@ -711,16 +727,87 @@ panel_applet_frame_cnx_broken (PanelAppletFrame *frame)
 				GTK_MESSAGE_QUESTION,
 				GTK_BUTTONS_YES_NO,
 				txt);
+	gtk_window_set_screen (GTK_WINDOW (dialog), screen);
 
 	g_signal_connect (dialog, "response",
 			  G_CALLBACK (panel_applet_frame_reload_response),
-			  frame);
+			  g_object_ref (frame));
 
 	gtk_widget_show (dialog);
 	g_free (applet_name);
 	g_free (txt);
 }
 
+static inline void
+register_stock_item (void)
+{
+        static gboolean registered = FALSE;
+
+        if (!registered) {
+                GtkIconFactory      *factory;
+                GtkIconSet          *cancel_icons;
+
+                static GtkStockItem  dont_delete_item [] = {
+                        { PANEL_STOCK_DONT_DELETE, N_("D_on't Delete"), 0, 0, GETTEXT_PACKAGE },
+                };
+
+                cancel_icons = gtk_icon_factory_lookup_default (GTK_STOCK_CANCEL);
+
+                factory = gtk_icon_factory_new ();
+
+                gtk_icon_factory_add (factory, PANEL_STOCK_DONT_DELETE, cancel_icons);
+
+                gtk_icon_factory_add_default (factory);
+
+                gtk_stock_add_static (dont_delete_item, 1);
+
+                registered = TRUE;
+        }
+}
+
+static void
+panel_applet_frame_loading_failed (PanelAppletFrame  *frame,
+				   CORBA_Environment *ev,
+				   const char        *iid,
+				   const char        *gconf_key,
+				   GtkWindow         *panel)
+{
+	GtkWidget *dialog;
+	char      *error;
+	int        response;
+
+	error = bonobo_exception_get_text (ev);
+
+	dialog = gtk_message_dialog_new (
+				NULL, 0,
+				GTK_MESSAGE_QUESTION,
+				GTK_BUTTONS_NONE,
+				_("The panel encountered a problem while loading \"%s\"\n"
+				  "Details: %s\n\n"
+				  "Do you want to delete the applet from your configuration?"),
+				iid, error);
+
+	g_free (error);
+
+	register_stock_item ();
+
+	gtk_dialog_add_buttons (GTK_DIALOG (dialog),
+				PANEL_STOCK_DONT_DELETE, GTK_RESPONSE_CANCEL,
+				GTK_STOCK_DELETE, GTK_RESPONSE_OK,
+				NULL);
+	gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
+
+	gtk_window_set_screen (GTK_WINDOW (dialog),
+			       gtk_window_get_screen (panel));
+
+        response = gtk_dialog_run (GTK_DIALOG (dialog));
+
+	gtk_widget_destroy (dialog);
+
+	if (response == GTK_RESPONSE_OK)
+		panel_applet_clean_gconf (APPLET_BONOBO, gconf_key, TRUE);
+}
+
 static void
 panel_applet_frame_class_init (PanelAppletFrameClass *klass,
 			       gpointer               dummy)
@@ -751,7 +838,6 @@ panel_applet_frame_instance_init (PanelAppletFrame      *frame,
 	frame->priv->orient           = PANEL_ORIENT_UP;
 	frame->priv->applet_info      = NULL;
 	frame->priv->moving_focus_out = FALSE;
-	frame->priv->clean_remove     = FALSE;
 }
 
 GType
@@ -878,7 +964,8 @@ panel_applet_frame_construct_moniker (PanelAppletFrame *frame,
 	char *retval;
 	char *bg_str;
 
-	bg_str = panel_applet_frame_get_background_string (frame, panel, panel->back_type);
+	bg_str = panel_applet_frame_get_background_string (
+				frame, panel, panel->background.type);
 
 	retval = g_strdup_printf (
 			"%s!prefs_key=/apps/panel/profiles/%s/applets/%s/prefs;"
@@ -916,17 +1003,9 @@ panel_applet_frame_construct (PanelAppletFrame *frame,
 	g_free (moniker);
 
 	if (BONOBO_EX (&ev)) {
-		char *err = bonobo_exception_get_text (&ev);
-		panel_error_dialog
-			("problem_loading_applet",
-			 _("<b>There was a problem loading applet '%s'</b>\n\n"
-			   "Details: %s"),
-			  iid, err);
-
+		panel_applet_frame_loading_failed (
+			frame, &ev, iid, gconf_key, GTK_WINDOW (panel->panel_parent));
 		CORBA_exception_free (&ev);
-
-		g_free (err);
-
 		return NULL;
 	}
 
