@@ -4,6 +4,7 @@
 #include <gtk/gtk.h>
 
 #include "panel-shell.h"
+#include "panel-util.h"
 
 struct _PanelShellPrivate {
 	int dummy;
@@ -51,10 +52,16 @@ panel_shell_register (void)
 		}
 
 		if (message) {
-			GtkWidget *dlg = panel_error_dialog
-				("panel_shell_register_error", "%s", message);
-			gtk_window_set_modal (GTK_WINDOW (dlg), TRUE);
+			GtkWidget *dlg = panel_error_dialog (
+						gdk_screen_get_default (),
+						"panel_shell_register_error",
+						"%s", message);
+
+			/* FIXME: quick hack */
+			g_signal_handlers_disconnect_by_func
+				(dlg, G_CALLBACK (gtk_widget_destroy), dlg);
 			gtk_dialog_run (GTK_DIALOG (dlg));
+			gtk_widget_destroy (dlg);
 			return FALSE;
 		}
 	}
